%% demo chemotax, quasilinear system, uses getKuvd to get \pa_u(div(c(u)grad v))
%% Init
close all; lx=0.5; ly=2; nx=10; ny=round(ly*nx/lx); 
p=[]; p=cheminit(p,lx,ly,nx,ny); p.sw.bifcheck=2; p.nc.mu2=0.05;
p=setfn(p,'tr'); p.sw.spcalc=1; p.sw.verb=2; p=cont(p,30);
%% bif.branches, switching off bifdetec for speed 
p=swibra('tr','bpt1','q1',0.1); 
p.sw.bifcheck=0; p.sw.spcalc=0; p.nc.dsmax=0.5; p=cont(p,30); 
%% 2 stripes 
p=swibra('tr','bpt2','q2',0.5); 
p.sw.bifcheck=0; p.sw.spcalc=0; p=cont(p,35);
%% BP3, multiple 
aux=[]; aux.soltol=1e-20; aux.isotol=1e-12; aux.besw=0; aux.m=3;  
p0=cswibra('tr','bpt3',aux); p0.sw.bifcheck=0; p0.nc.tol=1e-6; p0.sw.spcalc=0;
%% 2 spots on one side 
p=gentau(p0,1,'q3'); p.sol.ds=0.05; p=cont(p,30);
%% single stripe, makes a tiny loop
p=gentau(p0,[0 0 1],'q4'); p.sol.ds=0.05; p=cont(p,10);
%% 2 spots, with a  loop along the way 
p=swibra('tr','bpt5','q5',0.1); p.sw.bifcheck=0; p.sw.spcalc=0; p=cont(p,40); 
%% 3 spots 
p=swibra('tr','bpt6','q6',0.1); p.sw.bifcheck=0; p.sw.spcalc=0; p=cont(p,25); 
%% BP7, multiple subsequently use gentau
aux=[]; aux.soltol=1e-20; aux.isotol=1e-12; aux.besw=0; aux.m=2;  
p0=cswibra('tr','bpt7',aux); p0.sw.bifcheck=0; p0.nc.tol=1e-6; p0.sw.spcalc=0;
%% 4 spots
p=gentau(p0,[0 1],'q7'); p.sol.ds=0.05; p=cont(p,25);
%% 3 stripes
p=gentau(p0,1,'q8'); p.sol.ds=0.05; p=cont(p,25);
%% plot bifurcation diagram
figure(4);clf(4);cmp=1;
plotbra('tr',4,cmp,'cl','k'); 
plotbra('q1',4,cmp,'cl',p2pc('b1'),'lab',20,'lp',40);
plotbra('q2',4,cmp,'cl',p2pc('b2'),'lab',10); 
plotbra('q3',4,cmp,'cl',p2pc('r1'),'lab',20);
plotbra('q4',4,cmp,'cl',p2pc('b3'),'lp',20); %'lab',10,
plotbra('q5',4,cmp,'cl',p2pc('r2'),'lab',30); 
plotbra('q6',4,cmp,'cl',p2pc('r3'),'lab',20);
plotbra('q7',4,cmp,'cl',p2pc('o1'),'lab',10);
plotbra('q8',4,cmp,'cl',p2pc('b3')); %,'lab',10);
axis([10 25 0 0.8]);xlabel('\lambda');
ylabel('||u_1-1||_{L^1}/|\Omega|');
%% plot some solns 
psol('q1','pt20',1,1,2);pause; psol('q2','pt10',1,1,2); pause 
psol('q3','pt20',1,1,2);pause; psol('q4','pt10',1,1,2);pause; 
psol('q5','pt30',1,1,2); pause; psol('q6','pt20',1,1,2); pause
psol('q7','pt10',1,1,2);pause; psol('q8','pt10',1,1,2); 
%% check the diff matrices 
n=p.np; plotsol(p,1,1,1); ux=p.u; 
ux(1:n)=p.mat.Dy*p.u(1:n); %ux(1:n)=smu(p,ux(1:n)); %ux(1:n)=p.mat.M(1:n,1:n)\ux(1:n); 
plotsolu(p,ux,3,1,1); 
%% check Jacobians
p.nc.del=1e-6; 
[Gu,Gn]=jaccheck(p); Gd=abs(Gu-Gn); e1=max(max(Gd)); figure(4); clf; spy(Gd>e1/2);